@@ -1,6 +1,8 @@
 """ hook specifications for pytest plugins, invoked from main.py and builtin plugins.  """
 
 from pluggy import HookspecMarker
+from .deprecated import PYTEST_NAMESPACE
+
 
 hookspec = HookspecMarker("pytest")
 
@@ -22,10 +24,9 @@ def pytest_addhooks(pluginmanager):
     """
 
 
-@hookspec(historic=True)
+@hookspec(historic=True, warn_on_impl=PYTEST_NAMESPACE)
 def pytest_namespace():
     """
-    (**Deprecated**) this hook causes direct monkeypatching on pytest, its use is strongly discouraged
     return dict of name->object to be made globally available in
     the pytest namespace.
 
@@ -33,6 +34,19 @@ def pytest_namespace():
 
     .. note::
         This hook is incompatible with ``hookwrapper=True``.
+
+    .. warning::
+        This hook has been **deprecated** and will be removed in pytest 4.0.
+
+        Plugins whose users depend on the current namespace functionality should prepare to migrate to a
+        namespace they actually own.
+
+        To support the migration its suggested to trigger ``DeprecationWarnings`` for objects they put into the
+        pytest namespace.
+
+        An stopgap measure to avoid the warning is to monkeypatch the ``pytest`` module, but just as the
+        ``pytest_namespace`` hook this should be seen as a temporary measure to be removed in future versions after
+        an appropriate transition period.
     """
 
 
@@ -98,6 +112,7 @@ def pytest_configure(config):
     :arg _pytest.config.Config config: pytest config object
     """
 
+
 # -------------------------------------------------------------------------
 # Bootstrapping hooks called for plugins registered early enough:
 # internal and 3rd party plugins.
@@ -163,6 +178,7 @@ def pytest_load_initial_conftests(early_config, parser, args):
 # collection hooks
 # -------------------------------------------------------------------------
 
+
 @hookspec(firstresult=True)
 def pytest_collection(session):
     """Perform the collection protocol for the given session.
@@ -220,6 +236,7 @@ def pytest_collect_file(path, parent):
     :param str path: the path to collect
     """
 
+
 # logging hooks for collection
 
 
@@ -245,6 +262,7 @@ def pytest_make_collect_report(collector):
 
     Stops at first non-None result, see :ref:`firstresult` """
 
+
 # -------------------------------------------------------------------------
 # Python test function related hooks
 # -------------------------------------------------------------------------
@@ -291,6 +309,7 @@ def pytest_make_parametrize_id(config, val, argname):
     :param str argname: the automatic parameter name produced by pytest
     """
 
+
 # -------------------------------------------------------------------------
 # generic runtest related hooks
 # -------------------------------------------------------------------------
@@ -382,6 +401,7 @@ def pytest_runtest_logreport(report):
     """ process a test setup/call/teardown report relating to
     the respective phase of executing a test. """
 
+
 # -------------------------------------------------------------------------
 # Fixture related hooks
 # -------------------------------------------------------------------------
@@ -407,6 +427,7 @@ def pytest_fixture_post_finalizer(fixturedef, request):
     the fixture result cache ``fixturedef.cached_result`` can
     still be accessed."""
 
+
 # -------------------------------------------------------------------------
 # test session related hooks
 # -------------------------------------------------------------------------
@@ -439,6 +460,7 @@ def pytest_unconfigure(config):
 # hooks for customizing the assert methods
 # -------------------------------------------------------------------------
 
+
 def pytest_assertrepr_compare(config, op, left, right):
     """return explanation for comparisons in failing assert expressions.
 
@@ -450,6 +472,7 @@ def pytest_assertrepr_compare(config, op, left, right):
     :param _pytest.config.Config config: pytest config object
     """
 
+
 # -------------------------------------------------------------------------
 # hooks for influencing reporting (invoked from _pytest_terminal)
 # -------------------------------------------------------------------------
@@ -503,7 +526,17 @@ def pytest_terminal_summary(terminalreporter, exitstatus):
 
 @hookspec(historic=True)
 def pytest_logwarning(message, code, nodeid, fslocation):
-    """ process a warning specified by a message, a code string,
+    """
+    .. deprecated:: 3.8
+
+        This hook is will stop working in a future release.
+
+        pytest no longer triggers this hook, but the
+        terminal writer still implements it to display warnings issued by
+        :meth:`_pytest.config.Config.warn` and :meth:`_pytest.nodes.Node.warn`. Calling those functions will be
+        an error in future releases.
+
+    process a warning specified by a message, a code string,
     a nodeid and fslocation (both of which may be None
     if the warning is not tied to a particular node/location).
 
@@ -511,6 +544,31 @@ def pytest_logwarning(message, code, nodeid, fslocation):
         This hook is incompatible with ``hookwrapper=True``.
     """
 
+
+@hookspec(historic=True)
+def pytest_warning_captured(warning_message, when, item):
+    """
+    Process a warning captured by the internal pytest warnings plugin.
+
+    :param warnings.WarningMessage warning_message:
+        The captured warning. This is the same object produced by :py:func:`warnings.catch_warnings`, and contains
+        the same attributes as the parameters of :py:func:`warnings.showwarning`.
+
+    :param str when:
+        Indicates when the warning was captured. Possible values:
+
+        * ``"config"``: during pytest configuration/initialization stage.
+        * ``"collect"``: during test collection.
+        * ``"runtest"``: during test execution.
+
+    :param pytest.Item|None item:
+        **DEPRECATED**: This parameter is incompatible with ``pytest-xdist``, and will always receive ``None``
+        in a future release.
+
+        The item being executed if ``when`` is ``"runtest"``, otherwise ``None``.
+    """
+
+
 # -------------------------------------------------------------------------
 # doctest hooks
 # -------------------------------------------------------------------------
@@ -522,6 +580,7 @@ def pytest_doctest_prepare_content(content):
 
     Stops at first non-None result, see :ref:`firstresult` """
 
+
 # -------------------------------------------------------------------------
 # error handling and internal debugging hooks
 # -------------------------------------------------------------------------
